import { LitElement, css } from 'lit';
import { property } from 'lit/decorators.js';
import { SelectableMixin } from '@umbraco-ui/uui-base/lib/mixins';
import { UUIEvent } from '@umbraco-ui/uui-base/lib/events';

class UUIRefEvent extends UUIEvent {
}
UUIRefEvent.OPEN = "open";
UUIRefEvent.SELECTED = "selected";
UUIRefEvent.UNSELECTED = "unselected";

var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
typeof require !== "undefined" ? require : (x) => {
  throw new Error('Dynamic require of "' + x + '" is not supported');
};
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp(target, key, result);
  return result;
};
class UUIRefElement extends SelectableMixin(LitElement) {
  constructor() {
    super();
    this.disabled = false;
    this.error = false;
    this.addEventListener("click", this.toggleSelect);
    this.addEventListener("keydown", this.handleSelectKeydown);
  }
  toggleSelect() {
    if (this.selectable === false)
      return;
    this.selected = !this.selected;
    this.dispatchEvent(new UUIRefEvent(this.selected ? UUIRefEvent.SELECTED : UUIRefEvent.UNSELECTED));
  }
  select() {
    this.selected = true;
  }
  deselect() {
    this.selected = false;
  }
  handleSelectKeydown(e) {
    if (e.key !== " " && e.key !== "Enter")
      return;
    e.preventDefault();
    this.toggleSelect();
  }
  handleOpenClick(e) {
    e.stopPropagation();
    this.dispatchEvent(new UUIRefEvent(UUIRefEvent.OPEN));
  }
  handleOpenKeydown(e) {
    if (e.key !== " " && e.key !== "Enter")
      return;
    e.preventDefault();
    e.stopPropagation();
    this.dispatchEvent(new UUIRefEvent(UUIRefEvent.OPEN));
  }
}
UUIRefElement.styles = [
  css`
      :host {
        position: relative;
        display: flex;
        justify-content: space-between;
        align-items: center;
        width: 100%;

        box-sizing: border-box;
        border-radius: var(--uui-border-radius,3px);
        background-color: var(--uui-interface-surface,#fefefe);
        --uui-card-before-opacity: 0;
        transition: --uui-card-before-opacity 120ms;
      }

      :host(:focus) {
        /** TODO: implement focus outline. */
        outline-color: #6ab4f0;
      }

      :host([error]) {
        border: 2px solid var(--uui-look-danger-border,#d42054);
        box-shadow: 0 0 4px 0 var(--uui-look-danger-border,#d42054),
          inset 0 0 2px 0 var(--uui-look-danger-border,#d42054);
      }

      :host([border]) {
        border: 1px solid var(--uui-interface-border,#c4c4c4);
      }

      :host([selectable]) {
        cursor: pointer;
      }
      :host([selectable]) #select-border {
        position: absolute;
        z-index: 2;
        top: -2px;
        left: -2px;
        right: -2px;
        bottom: -2px;
        pointer-events: none;
        opacity: 0;
        transition: opacity 120ms;
      }
      :host([selectable]) #select-border::after {
        content: '';
        position: absolute;
        width: 100%;
        height: 100%;
        box-sizing: border-box;
        border: 2px solid var(--uui-interface-select,#3544b1);
        border-radius: calc(var(--uui-border-radius,3px) + 2px);
        box-shadow: 0 0 4px 0 var(--uui-interface-select,#3544b1),
          inset 0 0 2px 0 var(--uui-interface-select,#3544b1);
      }
      :host([selected]) #select-border {
        opacity: 1;
      }
      :host([selectable]:not([selected]):hover) #select-border {
        opacity: 0.33;
      }
      :host([selectable][selected]:hover) #select-border {
        opacity: 0.8;
      }

      :host([selectable]:not([selected])) #open-part:hover + #select-border {
        opacity: 0;
      }
      :host([selectable][selected]) #open-part:hover + #select-border {
        opacity: 1;
      }

      :host([selectable]:not([selected]):hover) #select-border::after {
        -webkit-animation: not-selected--hover 1.2s infinite;
                animation: not-selected--hover 1.2s infinite;
      }
      @-webkit-keyframes not-selected--hover {
        0%,
        100% {
          opacity: 0.66;
        }
        50% {
          opacity: 1;
        }
      }
      @keyframes not-selected--hover {
        0%,
        100% {
          opacity: 0.66;
        }
        50% {
          opacity: 1;
        }
      }

      :host([selectable][selected]:hover) #select-border::after {
        -webkit-animation: selected--hover 1.4s infinite;
                animation: selected--hover 1.4s infinite;
      }
      @-webkit-keyframes selected--hover {
        0%,
        100% {
          opacity: 1;
        }
        50% {
          opacity: 0.66;
        }
      }
      @keyframes selected--hover {
        0%,
        100% {
          opacity: 1;
        }
        50% {
          opacity: 0.66;
        }
      }
      :host([selectable]) #open-part:hover + #select-border::after {
        -webkit-animation: none;
                animation: none;
      }

      button {
        font-size: inherit;
        font-family: inherit;
        border: 0;
        padding: 0;
        background-color: transparent;
        text-align: left;
      }

      slot[name='actions'] {
        display: flex;
        align-items: center;
        --uui-button-height: calc(var(--uui-size-2,6px) * 4);
        margin-right: var(--uui-size-2,6px);
      }
      #actions-container {
        opacity: 0;
        transition: opacity 120ms;
      }
      :host(:hover) #actions-container {
        opacity: 1;
      }

      :host([border]:not([disabled]):hover) {
        border-color: var(--uui-interface-border-hover,rgb(179, 179, 179));
      }

      :host([disabled]) #open-part {
        cursor: default;
      }

      :host([border][disabled]) {
        border-color: var(--uui-interface-border-disabled,rgb(189, 189, 189));
      }

      slot[name='tag'] {
        flex-grow: 1;

        display: flex;
        justify-content: flex-end;
        align-items: center;
      }
    `
];
__decorateClass([
  property({ type: Boolean, reflect: true })
], UUIRefElement.prototype, "disabled", 2);
__decorateClass([
  property({ type: Boolean, reflect: true })
], UUIRefElement.prototype, "error", 2);

export { UUIRefElement };
